/**
 * The Seeks proxy and plugin framework are part of the SEEKS project.
 * Copyright (C) 2010, 2011 Emmanuel Benazera <ebenazer@seeks-project.info>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef QUERY_CAPTURE_H
#define QUERY_CAPTURE_H

#define NO_PERL

#include "plugin.h"
#include "interceptor_plugin.h"
#include "sp_exception.h"
#include "query_capture_configuration.h"
#include "DHTKey.h"
#include "search_snippet.h"
#include "user_db.h"

using namespace sp;
using dht::DHTKey;

namespace seeks_plugins
{

  class query_db_sweepable : public user_db_sweepable
  {
    public:
      query_db_sweepable();

      virtual ~query_db_sweepable();

      virtual bool sweep_me();

      virtual int sweep_records();

      time_t _last_sweep;
  };

  class query_capture_element;

  class query_capture : public plugin
  {
    public:
      query_capture();

      virtual ~query_capture();

      virtual void start();

      virtual void stop();

      static sp_err cgi_qc_redir(client_state *csp,
                                 http_response *rsp,
                                 const hash_map<const char*, const char*, hash<const char*>, eqstr> *parameters);

      static sp_err qc_redir(client_state *csp,
                             http_response *rsp,
                             const hash_map<const char*, const char*, hash<const char*>, eqstr> *parameters,
                             char *&urlp);

      virtual sp::db_record* create_db_record();

      int remove_all_query_records();

      static void store_queries(const std::string &query,
                                const query_context *qc,
                                const std::string &url, const std::string &host
                               ) throw (sp_exception);

      // store_query called from websearch plugin.
      void store_queries(const std::string &query) const throw (sp_exception);

      static void process_url(std::string &url, std::string &host);

      static void process_url(std::string &url, std::string &host, std::string &path);

      static void process_get(std::string &get);

      query_capture_element *_qelt;
  };

  class query_capture_element
  {
    public:
      query_capture_element();

      ~query_capture_element();

      /**
       * \brief stores queries halo along with a URL.
       */
      static void store_queries(const std::string &query,
                                const query_context *qc,
                                const std::string &url, const std::string &host,
                                const std::string &plugin_name,
                                const int &radius=-1) throw (sp_exception);

      /**
       * \brief stores queries halo alone.
       */
      static void store_queries(const std::string &query,
                                const std::string &plugin_name,
                                const int &radius=-1) throw (sp_exception);

      /**
       * \brief removes a queries halo and all attached URLs.
       */
      static void remove_queries(const std::string &query,
                                 const std::string &plugin_name,
                                 const int &radius=-1) throw (sp_exception);

      /**
       * \brief store query alone.
       */
      static void store_query(const DHTKey &key,
                              const std::string &query,
                              const uint32_t &radius,
                              const std::string &plugin_name) throw (sp_exception);

      /**
       * \brief removes query and attached URLs.
       */
      static void remove_query(const DHTKey &key,
                               const std::string &query,
                               const uint32_t &radius,
                               const std::string &plugin_name) throw (sp_exception);

      /**
       * \brief store query and URL.
       */
      static void store_url(const DHTKey &key, const std::string &query,
                            const std::string &url, const std::string &host,
                            const uint32_t &radius,
                            const std::string &plugin_name,
                            const search_snippet *sp=NULL) throw (sp_exception);

      /**
       * \brief removes a URL attached to a query.
       */
      static void remove_url(const DHTKey &key, const std::string &query,
                             const std::string &url, const std::string &host,
                             const short &url_hits, const uint32_t &radius,
                             const std::string &plugin_name) throw (sp_exception);

      static void get_useful_headers(const std::list<const char*> &headers,
                                     std::string &host, std::string &referer,
                                     std::string &get, std::string &base_url);

      static std::string no_command_query(const std::string &query);

    public:
      query_db_sweepable _qds;

      static std::string _capt_filename; // pattern capture file.
      static std::string _cgi_site_host; // default local cgi address.
  };

} /* end of namespace. */

#endif
