/**
 * The Seeks proxy and plugin framework are part of the SEEKS project.
 * Copyright (C) 2010, 2011 Emmanuel Benazera, ebenazer@seeks-project.info
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef QUERY_CAPTURE_CONFIGURATION_H
#define QUERY_CAPTURE_CONFIGURATION_H

#include "configuration_spec.h"

using sp::configuration_spec;

namespace seeks_plugins
{

  class query_capture_configuration : public configuration_spec
  {
    public:
      query_capture_configuration(const std::string &filename);

      ~query_capture_configuration();

      // virtual
      virtual void set_default_config();

      virtual void handle_config_cmd(char *cmd, const uint32_t &cmd_hash, char *arg,
                                     char *buf, const unsigned long &linenum);

      virtual void finalize_configuration();

      // main options.
      short _max_radius; /**< query LSH generation max radius. */
      time_t _sweep_cycle; /**< how long between two cycles of query db record sweeping. */
      time_t _retention;   /**< query db record retention, in seconds. */
      bool _protected_redirection; /**< whether URL redirection is protected against abuses. */
      bool _save_url_data; /**< whether to save URL title & summary for reuse. */
      std::string _cross_post_url; /**< default URL to which to cross-post recommendations. */

      static query_capture_configuration *_config;
  };

} /* end of namespace. */

#endif
